" Vim filetype plugin file
" Language:	TeX
" Maintainer:	Giuseppe Bilotta <bourbaki@bigfoot.com>
" Last change:	2001 Sep 19

" Only do this when not done yet for this buffer
if exists("b:did_ftplugin")
  finish
endif

" Don't load another plugin for this buffer
let b:did_ftplugin = 1

" Run default plug-in
" so $VIMRUNTIME/ftplugin/tex.vim

" If the format has been set, end
if exists("b:tex_format")
  finish
endif

" Now check for format. Default is plain.
let b:tex_format = 'plain'

let s:line = getline(1)

" If the file contains the Web2C %& shebang, get it:
if s:line =~ '^%&[a-z]\+'
  let b:tex_format = strpart(s:line, 2)

" Else, try and look for something. First of all, skip all
" initial comments:
else
  let s:linenum = 1
  while s:line =~ '%.'
    let s:linenum = s:linenum + 1
    let s:line = getline(s:linenum)
  endw
  if s:line =~ '\\documentclass'
    let b:tex_format = 'latex'
  elseif s:line =~ '\\documentstyle'
    let b:tex_format = 'latex209'
  elseif s:line =~ '\\\(starttekst\|stoptekst\|startonderdeel\|startoverzicht\)'
    let b:tex_format = 'cont-nl'
  elseif s:line =~ '\\\(starttext\|startcomponent\|startoverview\)'
    let b:tex_format = 'cont-en'
  elseif s:line =~ '\\\(stelle\|verwende\|umgebung\|benutze\)'
    let b:tex_format = 'cont-de'
  elseif s:line =~ '\\\(stel\|gebruik\|omgeving\)'
    let b:tex_format = 'cont-nl'
  elseif s:line =~ '\\\(use\|setup\|environment\)'
    let b:tex_format = 'cont-en'
  elseif s:line =~ '\\\(usa\|imposta\|ambiente\|inizia\)'
    let b:tex_format = 'cont-it'
  endif
endif

setlocal tw=66 fo-=l fo+=tcro1
setlocal comments=:%
setlocal include='\\input'
setlocal suffixesadd=.tex

if b:tex_format =~ 'latex'
  setlocal include='\\\(input\|include\)'
  setlocal suffixesadd=.tex,.ltx,.latex
endif

if b:tex_format =~ 'cont-'
  setlocal comments=b:%D,b:%S,b:%C,b:%I,:%
endif

setlocal ts=4 et

if b:tex_format =~ 'cont-'
  call SetTeXMenu()
  au BufLeave % call UnsetTeXMenu()

  setlocal keywordprg=texshow
"  let s:mod_path = substitute(expand("<sfile>:p:h"), '\\', '/', 'g')
"  exe "perl use lib '" s:mod_path "'"
  perl use lib 'c:/vim/vimfiles/ftplugin'
"  perl VIM::Msg(join(' ', @INC))
  perl use vim_tsw
  perl vim_tsw::setups_found
  perl vim_tsw::load_setups
"  perl sub texshow {$tex_cmd = shift ; vim_tsw::show_setup "$tex_cmd" }

  func! TeXShow()
    " Decide string to look for
    let tsw_txt = expand("<cword>")
    
    if tsw_txt != ""
      " Create scratch window
      new
      set buftype=nofile
      set noswapfile
      exe 'perl vim_tsw::show_setup "' . tsw_txt . '"'
      wincmd p
    else
      echo "Nothing to look for"
    endif
  endfun
endif

" Add mappings, unless the user didn't want this.
if !exists("no_plugin_maps") && !exists("no_tex_maps")
  imap <buffer>  \`a
  imap <buffer>  \`e
  imap <buffer>  \'e
  imap <buffer>  \`{\i}
  imap <buffer>  \`o
  imap <buffer>  \`u
  if b:tex_format =~ 'cont-'
    nmap <S-F1> :call TeXShow()<CR>
    imap <S-F1> <C-O>:call TeXShow()<CR>
  endif
endif

echo 'TeX format detected:' b:tex_format

