#D \module
#D   [       file=vim\_tsw.pm,
#D        version=2001.07.18,
#D          title=Perl/VIM modules, 
#D       subtitle=showing \CONTEXT\ commands,
#D         author=Hans Hagen,
#D           date=\currentdate,
#D      copyright={PRAGMA / Hans Hagen \& Ton Otten},
#D    suggestions={Tobias Burnus \& Taco Hoekater}]
#C
#C This module is part of the \CONTEXT\ macro||package and is
#C therefore copyrighted by \PRAGMA. See licen-en.pdf for 
#C details. 

package vim_tsw ; 

use strict ;

#D We scan for available setup files, with suffix \type {tws}.
#D These should be somewhere on the system, grouped in one
#D directory. At least the english file \type {cont-en.tws}
#D should be found. 

my $tws_path        = '' ; 
my @setup_files     = ('cont-en.tws') ; 
my $setup_file      = $setup_files[0] ; 
my $setup_interface = 'en' ; 
my $old_setup_file  = '' ;

my $sw = "" ;

sub set_setup_interface 
  { $setup_interface = shift } 

sub setups_found  
  { $tws_path = "c:/localtexmf/context/data/"; # FIXME
    @setup_files = glob ("$tws_path*.tws") ; 
    if (@setup_files) 
      { foreach (@setup_files) { s/\.tws// ; s/.*\/// } 
        $setup_file = $setup_files[0] ;
        return 1 } 
    else 
      { return 0 } } 

my %setups ;
my %commands ;   
my %loadedsetups ; 
my %positions ; 
my %crosslinks ; 

my $current_setup  = '' ; 

sub load_setup 
  { my $filename = shift ; 
    unless (keys %{$commands{$filename}})
      { local $/ = 'stopsetup' ; # in plaats van '' ivm unix ; (taco)
        $current_setup = '' ; 
        if (open(SETUP, "$tws_path$filename.tws" ))
          { my $position = 0 ; 
            while (<SETUP>)
              { chomp  ;
                s/startsetup//mso ;
                s/stopsetup//mso ; # redundant geworden 
                s/\n\n //gms ;     # in plaats van s/ //gms ; (taco)
                s/com\:(.*?)\:\s(.*)//mso ;
                my $string = $1 ; 
                my $command = $1 ;
                my $setup = $2 ;
                ++$position ;
                $string =~ s/(.*?)\<\<(.*?)\>\>(.*?)/$1$2$3/o ;
                $setups    {$filename}{$string}   = $setup ;
                $commands  {$filename}{$string}   = $command ;
                $positions {$filename}{$string}   = $position ;
                $crosslinks{$filename}[$position] = $string }
            close (SETUP) } }
    my @list = sort {lc $a cmp lc $b} keys %{$commands{$filename}} } 

sub load_setups 
  { foreach my $setup (@setup_files) { load_setup ($setup) } } 

my @arguments      = () ;  
my $nested_setup   = 0  ;
my $continue_setup = 0  ;
my $argument       = 0  ;
my $stopsuffix     = '' ;
my $stopcommand    = '' ;

my %arg ;

$arg {repeat} = '//n*/'                   ;
$arg {arg}    = 'argument/{/.../}'        ;
$arg {args}   = 'argument/{/..,...,../}'  ;
$arg {dis}    = 'argument/$$/.../$$'      ;
$arg {idx}    = 'argument/{/.../}'        ;
$arg {idxs}   = 'argument/{/..+...+../}'  ;
$arg {mat}    = 'argument/$/...:$'        ;
$arg {nop}    = '//.../'                  ;
$arg {fil}    = '//.../'                  ;
$arg {pos}    = 'symbol/(/.../)'          ;
$arg {poss}   = 'symbol/(/...,.../)'      ;
$arg {sep}    = 'command//\\\\/'          ;
$arg {ref}    = 'symbol/[/ref/]'          ;
$arg {refs}   = 'symbol/[/ref,../]'       ;
$arg {val}    = 'symbol/[/.../]'          ;
$arg {vals}   = 'symbol/[/..,...,../]'    ;
$arg {var}    = 'symbol/[/..=../]'        ;
$arg {vars}   = 'symbol/[/..,..=..,../]'  ;
$arg {cmd}    = 'command//\cmd/'          ;
$arg {dest}   = 'symbol/[/..ref/]'        ;
$arg {dests}  = 'symbol/[/..,..refs,../]' ;
$arg {trip}   = 'symbol/[/x:y:z=/]'       ;
$arg {trips}  = 'symbol/[/x:y:z=,../]'    ;
$arg {wrd}    = 'argument/{/.../}'        ;
$arg {wrds}   = 'argument/{/......./}'    ;
$arg {par}    = 'command//\par/'          ;
$arg {stp}    = '//stop/'                 ;
$arg {par}    = 'command///'              ;

sub show_command
  { my $command = shift ;
    local $_ = $commands{$setup_file}{$command} ;
    if ($command eq $_)
      { $sw = "\\$command" }
    elsif (/(.*?)\<\<(.*?)\>\>(.*?)/o)
      { $sw = "\\" ;
        if ($1) { $sw .= $1 }
        if ($2) { $sw .= $2 } # With format "user"
        if ($3) { $sw .= $3 }
        $stopsuffix = $2 } }

sub show_left_argument
  { local $_ = shift ; 
    my @thearg = split (/\//, $arg{$arguments[$_]}) ;
    $sw .= $thearg[1] # With format $thearg[0]
  }

sub show_middle_argument
  { local $_ = shift ; 
    my @thearg = split (/\//, $arg{$arguments[$_]}) ;
    if ($thearg[1])
      { $sw .= $thearg[2] } 
    else
      { $sw .= $thearg[2] } # With format $thearg[0]
  }

sub show_right_argument
  { local $_ = shift ; 
    my @thearg = split (/\//, $arg{$arguments[$_]}) ;
    $sw .= $thearg[3] ; # With format $thearg[0]
    ++$argument }  

sub show_reference
  { if (($nested_setup<=1)&&(defined($arguments[$argument])))
      { if ($arguments[$argument]=~/ref/)
          { $sw .= "\n" ;
            show_left_argument ($argument) ;
            show_middle_argument ($argument) ;
            show_right_argument ($argument) } } }

sub show_stop_command
  { my $before_stop = shift ; 
    if ($stopcommand)
      { if ($stopsuffix)
          { $sw .= '\\stop' ;
            $sw .= $stopsuffix }  # With format "user"
        else
          { $sw .= $stopcommand } } }

sub show_whatever_left
  { while ($argument<@arguments) 
      { $sw .= "\n" ;
        show_left_argument ($argument) ;
        show_middle_argument ($argument) ;
        show_right_argument ($argument) ; 
        ++$argument } 
    if ($stopcommand)
      { $sw .= "\n...\n...\n...\n" ;
        show_stop_command } }

sub do_update_setup                # type: 0=all 1=vars 2=vals 
  { my ($command, $type) = @_ ; 
    my $setup = $setups{$setup_file}{$command} ;
    my $default = '' ; 
    my $key = '' ; 
    my $meaning = '' ; 
    my @values = () ; 
    local $_ ;
    ++$nested_setup ;
    while ($setup=~/(typ|var|val|ivr|ivl)\:(.*?)\:\s/mgo)
      { $key = $1 ;
        $meaning = $2 ;
        if    (($key=~/var/o)&&($type!=2))
          { $_ = $meaning ; s/(.*?)\:(.*?)\:(.*)//o ;
            if (($nested_setup>1)&&(!$2)) { next }
            $key = $1 ; 
            if ($3) { $default = $3 } else { $default = '' }
            $_= $2 ; s/\s//go ; @values = split (/,/,$_) ; 
            if ($continue_setup)
              { $sw .= ",\n " }
            else
              { $continue_setup = 1 ;
                $sw .= "\n" ;
                show_left_argument($argument) }
            $sw .= $key ;
            $sw .= '=' ; # With format "symbol"
            while (@values)
              { my $value = shift @values ;
                if ($value =~ /^\*/o) 
                  { $value =~ s/^\*//o ; 
                    $sw .= lc $value } # With format "variable"
                elsif ($value eq $default)
                  { $sw .= $value }
                else
                  { $sw .= $value }
                if (@values)
                  { $sw .= '|' } # With format "or"
                else
                  { last } } }
        elsif (($key=~/val/o)&&($type!=1))
          { $_ = $meaning ; s/(.*)\:(.*)//o ;
            if (($nested_setup>1)&&(!$2)) { next }
            $_ = $1 ; s/\s//go ; @values = split (/,/,$_) ;
            if ($2) { $default = $2 } else { $default = '' }
            if ($continue_setup)
              { $continue_setup = 0 ;
                show_right_argument($argument) }
            $sw .= "\n" ;
            show_left_argument($argument) ;
            while (@values)
              { unless (@values) { last } 
                my $value = shift (@values) ;
                if ($value =~ /^\*/o)
                  { $value =~ s/^\*//o ; 
                    $sw .= lc $value } # With format "variable"
                elsif ($value eq $default)
                  { $sw .= $value } # With format "default"
                else
                  { $sw .= $value }
                if (@values)
                  { $sw .= ', ' }
                else
                  { last } }
            show_right_argument($argument) }
        elsif ($key=~/typ/o)
         { if ($nested_setup==1)
             { show_command ($command) ;
               my $arguments = $meaning ;
               if ($arguments=~/stp/)
                 { $_ = $command ;
                   s/start(.*)/$1/o ;
                   $stopcommand = "\\stop$_" ;
                   $arguments =~ s/stp//go }
               @arguments = split (/\//,$arguments) ;
               if (@arguments)
                 { for (my $i=0;$i<@arguments;$i++)  
                    { show_left_argument ($i) ;
                      show_middle_argument ($i) ;
                      show_right_argument ($i) }
                   if ($stopcommand) 
                     { $sw .= ' ... ' ;
                       show_stop_command } 
                   $sw .= "\n\n" ;
                   show_command ($command) }
               $argument = 0 ;
               $continue_setup = 0 } }
        elsif ($key=~/ivr/o)
          { $meaning =~ s/(.*)\:(.*)//o ;
            do_update_setup ($1,1) }
        elsif ($key=~/ivl/o)
          { $meaning =~ s/(.*)\:(.*)//o ;
            do_update_setup ($1,2) }
        show_reference }
   --$nested_setup ;
   if (($continue_setup)&&(!$nested_setup)) 
     { show_right_argument ;
       show_whatever_left } }

sub show_setup 
  { my $asked_for = shift ;
    unless ($asked_for) { return } 
    my $found = 0 ;  
    $asked_for =~ s/^\\// ;    
    if ($setup_interface) 
      { $found = 0 ; 
        foreach my $name (@setup_files) 
          { if (($name=~/\-$setup_interface/)&&(exists($commands{$name}{$asked_for}))) 
              { $found = 1 ; 
                $setup_file = $name ; 
                last } } }  
    if (!($found)&&(exists($commands{$setup_file}{$asked_for}))) 
      { $found = 1 } 
    else 
      { $found = 0 ; 
        foreach my $name (@setup_files) 
          { if (exists($commands{$name}{$asked_for})) 
              { $found = 1 ; 
                $setup_file = $name ; 
                last } } }  
    if ($found) 
      { my @list = sort {lc $a cmp lc $b} keys %{$commands{$setup_file}} ;
        $found = 0 ;
        do_update_setup($asked_for,0) ;
	my @sw_array = split(/\n/,$sw) ;
	$main::curbuf->Append(0, @sw_array) } } 

1 ;

__END__

